#include "stock.h"

/* dump_companies() dumps all the files into company.dat, which can in turn
 * be read by the function init_companies()
 * both functions follow the same format, and hopefully one can read what
 * the other has written */

void dump_companies() {
   FILE *company_fp;
   int count;
   cid_t i = 0;
   company_t *c;
   
   company_fp = fopen ("company.dato", "w");
   if (company_fp == NULL){
      printf ("Can't open file for dumping company data\n");
      exit(1);
   }
   
   do {
      ++i;
      c = &company[i];
      
      fprintf (company_fp, "%d %d\n", c->cid, c->gid);
      /* putting the cid, and gid first */
      
      fprintf (company_fp, "%s, %s", 
	       c->name, c->index);
      
      /* next go the name, index and comment */

      for (count=0; count < max_simult_quot; ++count) {
	 stock_company_t *s;
	 
	 s = c->buy[count];
	 fprintf (company_fp, "%u %d %d\n", s->plid, s->pr, s->qty);
	 /* now the buy list, one at a time, max_simult_quot times */
      }
      
      fprintf (company_fp, "\n");
      
      for (count=0; count < max_simult_quot; ++count) {
	 stock_company_t *s;
	 
	 s = c->sell[count];	 
	 fprintf (company_fp, "%u %d %d\n", s->plid, s->pr, s->qty);
	 /* and the sell list, one at  a time max_simul_quot times */
      }
      fprintf (company_fp, "\n\n");
   } while (i < max_companies);
   
   fclose (company_fp);
}
	 
	 
/* init_companies() reads from a file "company.dat" in order to initialize
 * the company array. a bold step, but it hasn't been tested yet.
 * could do very well with a separate program to dump data to the file,
 * in order to start afresh... */

void init_companies () {
   FILE *company_fp;
   int count;
   company_t *c;

   company_fp = fopen ("company.dat", "r");
   if (company_fp == NULL){
      printf ("Can't open file for reading initial company info\n");
      exit(1);
   }
   
   do {
      
      if (max_companies % resize == 0)
	company = (company_t *) xrealloc (company, (max_companies + resize) * 
					  sizeof (company_t));
      
      c = &company[max_companies];
      
      /* read the cid, and gid in one shot */

      fscanf (company_fp, "%d %d", &c->cid, &c->gid);
#ifdef DEBUG
      printf ("%d %d\n", c->cid, c->gid);
#endif
      
      if (c->gid > max_groups) max_groups = c->gid;
      
      /* read the name, index in one shot */      
           
      c->name = (char *) xmalloc (32);
      c->index = (char *) xmalloc (4);
      
      fscanf (company_fp, "%*c %32[^,], %4s", c->name, c->index);
      /* This nice GCC feature could be used, if GCC is used to compile..
       * The only reason this is commented is for ANSI  compliance.
       * 
       * fscanf (company_fp, "%*c %a[^,], %as", &c->name, &c->index);
       * 
       */

      /* Read initial buy list */
      
      c->buy = (stock_company_t **) xmalloc (max_simult_quot * 
					     sizeof(stock_company_t *));
      
      for (count=0; count < max_simult_quot; count++) {
	 stock_company_t *s;
	 
	 c->buy[count] = (stock_company_t *) xmalloc (sizeof(stock_company_t));
	 s = c->buy[count];
	 fscanf (company_fp, "%d %d %d ", &s->plid, &s->pr, &s->qty);
      }
      
      /* Read initial sell list */
      
      c->sell = (stock_company_t **) xmalloc (max_simult_quot * 
					      sizeof(stock_company_t *));
      
      for (count=0; count < max_simult_quot; count++) {
	 stock_company_t *s;
	 
	 c->sell[count] = (stock_company_t *) xmalloc (sizeof(stock_company_t));
	 s = c->sell[count];
	 fscanf (company_fp, "%u %d %d ", &s->plid, &s->pr, &s->qty);
      }
      
      /* setup trade_history */      
      c->trade_history = (pr_t *) xmalloc (max_history * sizeof(pr_t));

      ++max_companies;
   } while (!feof(company_fp));
   
   --max_companies;   
   fclose (company_fp);
}
